@@ -201,7 +201,11 @@ private:
   enum_monotonicity_info m_part_func_monotonicity_info;
 public:
   handler *clone(const char *name, MEM_ROOT *mem_root);
+#ifndef MCP_BUG56438
   virtual void set_part_info(partition_info *part_info, bool early)
+#else
+  virtual void set_part_info(partition_info *part_info)
+#endif
   {
      m_part_info= part_info;
      m_is_sub_partitioned= part_info->is_sub_partitioned();
@@ -272,6 +276,10 @@ public:
     DBUG_RETURN(0);
   }
   virtual void change_table_ptr(TABLE *table_arg, TABLE_SHARE *share);
+  /*
+  virtual bool check_if_incompatible_data(HA_CREATE_INFO *create_info,
+                                          uint table_changes);
+  */
 private:
   int prepare_for_rename();
   int copy_partitions(ulonglong * const copied, ulonglong * const deleted);
@@ -515,13 +523,21 @@ public:
   virtual int read_range_next();
 
 private:
-  bool init_record_priority_queue();
-  void destroy_record_priority_queue();
   int common_index_read(uchar * buf, bool have_start_key);
   int common_first_last(uchar * buf);
   int partition_scan_set_up(uchar * buf, bool idx_read_flag);
   int handle_unordered_next(uchar * buf, bool next_same);
   int handle_unordered_scan_next_partition(uchar * buf);
+  uchar *queue_buf(uint part_id)
+    {
+      return (m_ordered_rec_buffer +
+              (part_id * (m_rec_length + PARTITION_BYTES_IN_POS)));
+    }
+  uchar *rec_buf(uint part_id)
+    {
+      return (queue_buf(part_id) +
+              PARTITION_BYTES_IN_POS);
+    }
   int handle_ordered_index_scan(uchar * buf, bool reverse_order);
   int handle_ordered_next(uchar * buf, bool next_same);
   int handle_ordered_prev(uchar * buf);
@@ -633,9 +649,12 @@ public:
     underlying handlers must have the same implementation for it to work.
   */
   virtual uint8 table_cache_type();
+  virtual ha_rows records();
+
+#ifndef MCP_BUG56438
   /* Calculate hash value for PARTITION BY KEY tables.  */
   uint32 calculate_key_hash_value(Field **field_array);
-  virtual ha_rows records();
+#endif
 
   /*
     -------------------------------------------------------------------------
@@ -889,6 +908,14 @@ public:
     return m_file[0]->index_flags(inx, part, all_parts);
   }
 
+  /**
+     wrapper function for handlerton alter_table_flags, since
+     the ha_partition_hton cannot know all its capabilities
+  */
+/*
+  virtual uint alter_table_flags(uint flags);
+*/
+
   /*
      extensions of table handler files
   */
@@ -1068,15 +1095,18 @@ public:
     -------------------------------------------------------------------------
     MODULE on-line ALTER TABLE
     -------------------------------------------------------------------------
-    These methods are in the handler interface but never used (yet)
+    These methods are in the handler interface (used by innodb-plugin)
     They are to be used by on-line alter table add/drop index:
     -------------------------------------------------------------------------
-    virtual ulong index_ddl_flags(KEY *wanted_index) const
-    virtual int add_index(TABLE *table_arg,KEY *key_info,uint num_of_keys);
-    virtual int drop_index(TABLE *table_arg,uint *key_num,uint num_of_keys);
-    -------------------------------------------------------------------------
   */
+ /*
 
+    virtual int add_index(TABLE *table_arg, KEY *key_info, uint num_of_keys);
+    virtual int prepare_drop_index(TABLE *table_arg, uint *key_num,
+				   uint num_of_keys);
+    virtual int final_drop_index(TABLE *table_arg);
+  */
+ 
   /*
     -------------------------------------------------------------------------
     MODULE tablespace support
