@@ -1,4 +1,5 @@
-/* Copyright (C) 2003 MySQL AB
+/*
+   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,7 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #ifndef DICT_TAB_INFO_HPP
 #define DICT_TAB_INFO_HPP
@@ -28,10 +30,10 @@
 // sql/my_decimal.h requires many more sql/*.h new to ndb
 // for now, copy the bit we need  TODO proper fix
 
-#define DECIMAL_MAX_LENGTH ((8 * 9) - 8)
+#define DECIMAL_MAX_FIELD_SIZE ((9 * 9) - (2 * 8))
 
-#ifndef NOT_FIXED_DEC
-#define NOT_FIXED_DEC                   31
+#ifndef DECIMAL_NOT_SPECIFIED
+#define DECIMAL_NOT_SPECIFIED 31
 #endif
 
 C_MODE_START
@@ -143,6 +145,14 @@ public:
     
     SingleUserMode     = 152,
 
+    HashMapObjectId    = 153,
+    HashMapVersion     = 154,
+
+    TableStorageType   = 155,
+
+    ExtraRowGCIBits    = 156,
+    ExtraRowAuthorBits = 157,
+
     TableEnd           = 999,
     
     AttributeName          = 1000, // String, Mandatory
@@ -160,8 +170,14 @@ public:
     AttributeExtScale      = 1015, //Default 0
     AttributeExtLength     = 1016, //Default 0
     AttributeAutoIncrement = 1017, //Default false
-    AttributeDefaultValue  = 1018, //Default value (printable string),
     AttributeArrayType     = 1019, //Default NDB_ARRAYTYPE_FIXED
+    AttributeDefaultValueLen = 1020, //Actual Length saved in AttributeDefaultValue
+    /* Default value (Binary type, not printable as string),
+     * For backward compatibility, the new keyValue
+     * (not use the old keyValue 1018) is added
+       when restoring data from low backup version data.
+    */
+    AttributeDefaultValue = 1021,
     AttributeEnd           = 1999  //
   };
   // ----------------------------------------------------------------------
@@ -180,8 +196,8 @@ public:
     DistrKeyHash = 4,
     DistrKeyLin = 5,
     UserDefined = 6,
-    DistrKeyUniqueHashIndex = 7,
-    DistrKeyOrderedIndex = 8
+    DistrKeyOrderedIndex = 8, // alias
+    HashMapPartition = 9
   };
   
   // TableType constants + objects
@@ -198,11 +214,15 @@ public:
     SubscriptionTrigger = 16,
     ReadOnlyConstraint = 17,
     IndexTrigger = 18,
+    ReorgTrigger = 19,
     
     Tablespace = 20,        ///< Tablespace
     LogfileGroup = 21,      ///< Logfile group
     Datafile = 22,          ///< Datafile
-    Undofile = 23           ///< Undofile
+    Undofile = 23,          ///< Undofile
+    HashMap = 24,
+
+    SchemaTransaction = 30
   };
 
   // used 1) until type BlobTable added 2) in upgrade code
@@ -253,7 +273,8 @@ public:
       tableType == HashIndexTrigger ||
       tableType == SubscriptionTrigger ||
       tableType == ReadOnlyConstraint ||
-      tableType == IndexTrigger;
+      tableType == IndexTrigger ||
+      tableType == ReorgTrigger;
   }
   static inline bool
   isFilegroup(int tableType) {
@@ -268,6 +289,12 @@ public:
       tableType == Datafile||
       tableType == Undofile;
   }
+
+  static inline bool
+  isHashMap(int tableType) {
+    return
+      tableType == HashMap;
+  }
   
   // Object state for translating from/to API
   enum ObjectState {
@@ -333,7 +360,7 @@ public:
     char   FrmData[MAX_FRM_DATA_SIZE];
     Uint32 FragmentCount;
     Uint32 ReplicaDataLen;
-    Uint16 ReplicaData[MAX_FRAGMENT_DATA_BYTES];
+    Uint16 ReplicaData[MAX_FRAGMENT_DATA_ENTRIES];
     Uint32 FragmentDataLen;
     Uint16 FragmentData[3*MAX_NDB_PARTITIONS];
 
@@ -345,13 +372,21 @@ public:
     Uint32 TablespaceDataLen;
     Uint32 TablespaceData[2*MAX_NDB_PARTITIONS];
     Uint32 RangeListDataLen;
-    char   RangeListData[4*2*MAX_NDB_PARTITIONS*2];
+    Uint32 RangeListData[2*MAX_NDB_PARTITIONS*2];
     
     Uint32 RowGCIFlag;
     Uint32 RowChecksumFlag;
 
     Uint32 SingleUserMode;
-    
+
+    Uint32 HashMapObjectId;
+    Uint32 HashMapVersion;
+
+    Uint32 TableStorageType;
+
+    Uint32 ExtraRowGCIBits;
+    Uint32 ExtraRowAuthorBits;
+
     Table() {}
     void init();
   };
@@ -414,7 +449,8 @@ public:
     Uint32 AttributeAutoIncrement;
     Uint32 AttributeStorageType;
     Uint32 AttributeDynamic;
-    char   AttributeDefaultValue[MAX_ATTR_DEFAULT_VALUE_SIZE];
+    Uint32 AttributeDefaultValueLen;  //byte sizes
+    Uint8  AttributeDefaultValue[MAX_ATTR_DEFAULT_VALUE_SIZE];
     
     Attribute() {}
     void init();
@@ -482,8 +518,9 @@ public:
           // copy from Field_new_decimal ctor
           uint precision = AttributeExtPrecision;
           uint scale = AttributeExtScale;
-          if (precision > DECIMAL_MAX_LENGTH || scale >= NOT_FIXED_DEC)
-            precision = DECIMAL_MAX_LENGTH;
+          if (precision > DECIMAL_MAX_FIELD_SIZE ||
+              scale >= DECIMAL_NOT_SPECIFIED)
+            return false;
           uint bin_size = my_decimal_get_binary_size(precision, scale);
           AttributeSize = DictTabInfo::an8Bit;
           AttributeArraySize = bin_size * AttributeExtLength;
@@ -568,8 +605,11 @@ public:
       fprintf(out, "AttributeExtPrecision = %d\n", AttributeExtPrecision);
       fprintf(out, "AttributeExtScale = %d\n", AttributeExtScale);
       fprintf(out, "AttributeExtLength = %d\n", AttributeExtLength);
-      fprintf(out, "AttributeDefaultValue = \"%s\"\n",
-        AttributeDefaultValue ? AttributeDefaultValue : "");
+      fprintf(out, "AttributeDefaultValueLen = %d\n",
+              AttributeDefaultValueLen);
+      fprintf(out, "AttributeDefaultValue: \n");
+      for (unsigned int i = 0; i < AttributeDefaultValueLen; i++)
+        fprintf(out, "0x%x", AttributeDefaultValue[i]);
     }
   };
   
@@ -597,7 +637,7 @@ private:
   Uint32 tabInfoData[DataLength];
 
 public:
-  enum Depricated
+  enum Deprecated
   {
     AttributeDGroup    = 1009, //Default NotDGroup
     AttributeStoredInd = 1011, //Default NotStored
@@ -735,4 +775,41 @@ struct DictFilegroupInfo {
   static const SimpleProperties::SP2StructMapping FileMapping[];
 };
 
+#define DHMIMAP(x, y, z) \
+  { DictHashMapInfo::y, my_offsetof(x, z), SimpleProperties::Uint32Value, 0, (~0), 0 }
+
+#define DHMIMAP2(x, y, z, u, v) \
+  { DictHashMapInfo::y, my_offsetof(x, z), SimpleProperties::Uint32Value, u, v, 0 }
+
+#define DHMIMAPS(x, y, z, u, v) \
+  { DictHashMapInfo::y, my_offsetof(x, z), SimpleProperties::StringValue, u, v, 0 }
+
+#define DHMIMAPB(x, y, z, u, v, l) \
+  { DictHashMapInfo::y, my_offsetof(x, z), SimpleProperties::BinaryValue, u, v, \
+                     my_offsetof(x, l) }
+
+#define DHMIBREAK(x) \
+  { DictHashMapInfo::x, 0, SimpleProperties::InvalidValue, 0, 0, 0 }
+
+struct DictHashMapInfo {
+  enum KeyValues {
+    HashMapName    = 1,
+    HashMapBuckets = 2,
+    HashMapValues  = 3
+  };
+
+  // Table data interpretation
+  struct HashMap {
+    char   HashMapName[MAX_TAB_NAME_SIZE];
+    Uint32 HashMapBuckets;
+    Uint16 HashMapValues[NDB_DEFAULT_HASHMAP_BUCKETS];
+    Uint32 HashMapObjectId;
+    Uint32 HashMapVersion;
+    HashMap() {}
+    void init();
+  };
+  static const Uint32 MappingSize;
+  static const SimpleProperties::SP2StructMapping Mapping[];
+};
+
 #endif
