@@ -1,7 +1,6 @@
 /** **/
 /*
-   Copyright (c) 2003-2008 MySQL AB, 2009, 2010 Sun Microsystems, Inc.
-   Use is subject to license terms.
+   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,6 +24,7 @@
 #endif
 
 #include <pc.hpp>
+#include <DynArr256.hpp>
 #include <SimulatedBlock.hpp>
 
 #ifdef DBACC_C
@@ -104,8 +104,6 @@ ndbout << "Ptr: " << ptr.p->word32 << " \tIndex: " << tmp_string << " \tValue: "
 #define ZDEFAULT_LIST 3
 #define ZWORDS_IN_PAGE 2048
 #define ZADDFRAG 0
-#define ZDIRARRAY 68
-#define ZDIRRANGESIZE 65
 //#define ZEMPTY_FRAGMENT 0
 #define ZFRAGMENTSIZE 64
 #define ZFIRSTTIME 1
@@ -140,7 +138,7 @@ ndbout << "Ptr: " << ptr.p->word32 << " \tIndex: " << tmp_string << " \tValue: "
 /**
  * Check kernel_types for other operation types
  */
-#define ZSCAN_OP 6
+#define ZSCAN_OP 8
 #define ZSCAN_REC_SIZE 256
 #define ZSTAND_BY 2
 #define ZTABLESIZE 16
@@ -155,7 +153,7 @@ ndbout << "Ptr: " << ptr.p->word32 << " \tIndex: " << tmp_string << " \tValue: "
 #define ZREL_ROOT_FRAG 5
 #define ZREL_FRAG 6
 #define ZREL_DIR 7
-#define ZREPORT_MEMORY_USAGE 8
+#define ZREL_ODIR 8
 
 /* ------------------------------------------------------------------------- */
 /* ERROR CODES                                                               */
@@ -182,6 +180,18 @@ ndbout << "Ptr: " << ptr.p->word32 << " \tIndex: " << tmp_string << " \tValue: "
 #define ZTO_OP_STATE_ERROR 631
 #define ZTOO_EARLY_ACCESS_ERROR 632
 #define ZDIR_RANGE_FULL_ERROR 633 // on fragment
+
+#if ZBUF_SIZE != ((1 << ZSHIFT_PLUS) - (1 << ZSHIFT_MINUS))
+#error ZBUF_SIZE != ((1 << ZSHIFT_PLUS) - (1 << ZSHIFT_MINUS))
+#endif
+
+static
+inline
+Uint32 mul_ZBUF_SIZE(Uint32 i)
+{
+  return (i << ZSHIFT_PLUS) - (i << ZSHIFT_MINUS);
+}
+
 #endif
 
 class ElementHeader {
@@ -272,6 +282,8 @@ ElementHeader::clearScanBit(Uint32 header, Uint32 scanBit){
 
 
 class Dbacc: public SimulatedBlock {
+  friend class DbaccProxy;
+
 public:
 // State values
 enum State {
@@ -304,24 +316,6 @@ enum State {
 
 // Records
 
-/* --------------------------------------------------------------------------------- */
-/* DIRECTORY RANGE                                                                   */
-/* --------------------------------------------------------------------------------- */
-  struct DirRange {
-    Uint32 dirArray[256];
-  }; /* p2c: size = 1024 bytes */
-  
-  typedef Ptr<DirRange> DirRangePtr;
-
-/* --------------------------------------------------------------------------------- */
-/* DIRECTORYARRAY                                                                    */
-/* --------------------------------------------------------------------------------- */
-struct Directoryarray {
-  Uint32 pagep[256];
-}; /* p2c: size = 1024 bytes */
-
-  typedef Ptr<Directoryarray> DirectoryarrayPtr;
-
 /* --------------------------------------------------------------------------------- */
 /* FRAGMENTREC. ALL INFORMATION ABOUT FRAMENT AND HASH TABLE IS SAVED IN FRAGMENT    */
 /*         REC  A POINTER TO FRAGMENT RECORD IS SAVED IN ROOTFRAGMENTREC FRAGMENT    */
@@ -355,7 +349,6 @@ struct Fragmentrec {
   Uint32 expReceiveIndex;
   Uint32 expReceiveForward;
   Uint32 expSenderDirIndex;
-  Uint32 expSenderDirptr;
   Uint32 expSenderIndex;
   Uint32 expSenderPageptr;
 
@@ -369,9 +362,8 @@ struct Fragmentrec {
 // in its turn references the pages) for the bucket pages and the overflow
 // bucket pages.
 //-----------------------------------------------------------------------------
-  Uint32 directory;
-  Uint32 dirsize;
-  Uint32 overflowdir;
+  DynArr256::Head directory;
+  DynArr256::Head overflowdir;
   Uint32 lastOverIndex;
 
 //-----------------------------------------------------------------------------
@@ -458,12 +450,10 @@ struct Fragmentrec {
 // hashcheckbit is the bit to check whether to send element to split bucket or not
 // k (== 6) is the number of buckets per page
 // lhfragbits is the number of bits used to calculate the fragment id
-// lhdirbits is the number of bits used to calculate the page id
 //-----------------------------------------------------------------------------
   Uint8 hashcheckbit;
   Uint8 k;
   Uint8 lhfragbits;
-  Uint8 lhdirbits;
 
 //-----------------------------------------------------------------------------
 // nodetype can only be STORED in this release. Is currently only set, never read
@@ -624,15 +614,16 @@ struct ScanRec {
 /* TABREC                                                                            */
 /* --------------------------------------------------------------------------------- */
 struct Tabrec {
-  Uint32 fragholder[MAX_FRAG_PER_NODE];
-  Uint32 fragptrholder[MAX_FRAG_PER_NODE];
+  Uint32 fragholder[MAX_FRAG_PER_LQH];
+  Uint32 fragptrholder[MAX_FRAG_PER_LQH];
   Uint32 tabUserPtr;
   BlockReference tabUserRef;
+  Uint32 tabUserGsn;
 };
   typedef Ptr<Tabrec> TabrecPtr;
 
 public:
-  Dbacc(Block_context&);
+  Dbacc(Block_context&, Uint32 instanceNumber = 0);
   virtual ~Dbacc();
 
   // pointer to TUP instance in this thread
@@ -640,6 +631,7 @@ public:
   class Dblqh* c_lqh;
 
   void execACCMINUPDATE(Signal* signal);
+  void removerow(Uint32 op, const Local_key*);
 
 private:
   BLOCK_DEFINES(Dbacc);
@@ -670,6 +662,11 @@ private:
   void execREAD_CONFIG_REQ(Signal* signal);
   void execDUMP_STATE_ORD(Signal* signal);
 
+  void execDROP_FRAG_REQ(Signal*);
+
+  void execDBINFO_SCANREQ(Signal *signal);
+  void execNODE_STATE_REP(Signal*);
+
   // Statement blocks
   void ACCKEY_error(Uint32 fromWhere);
 
@@ -685,10 +682,7 @@ private:
   void releaseFragResources(Signal* signal, Uint32 fragIndex);
   void releaseRootFragRecord(Signal* signal, RootfragmentrecPtr rootPtr);
   void releaseRootFragResources(Signal* signal, Uint32 tableId);
-  void releaseDirResources(Signal* signal,
-                           Uint32 fragIndex,
-                           Uint32 dirIndex,
-                           Uint32 startIndex);
+  void releaseDirResources(Signal* signal);
   void releaseDirectoryResources(Signal* signal,
                                  Uint32 fragIndex,
                                  Uint32 dirIndex,
@@ -700,8 +694,6 @@ private:
   void initScanFragmentPart(Signal* signal);
   Uint32 checkScanExpand(Signal* signal);
   Uint32 checkScanShrink(Signal* signal);
-  void initialiseDirRec(Signal* signal);
-  void initialiseDirRangeRec(Signal* signal);
   void initialiseFragRec(Signal* signal);
   void initialiseFsConnectionRec(Signal* signal);
   void initialiseFsOpRec(Signal* signal);
@@ -767,8 +759,11 @@ private:
   void increaselistcont(Signal* signal);
   void seizeLeftlist(Signal* signal);
   void seizeRightlist(Signal* signal);
-  Uint32 readTablePk(Uint32 localkey1, Uint32 eh, OperationrecPtr);
+  Uint32 readTablePk(Uint32 lkey1, Uint32 lkey2, Uint32 eh, OperationrecPtr);
   Uint32 getElement(Signal* signal, OperationrecPtr& lockOwner);
+  Uint32 getPagePtr(DynArr256::Head&, Uint32);
+  bool setPagePtr(DynArr256::Head& directory, Uint32 index, Uint32 ptri);
+  Uint32 unsetPagePtr(DynArr256::Head& directory, Uint32 index);
   void getdirindex(Signal* signal);
   void commitdelete(Signal* signal);
   void deleteElement(Signal* signal);
@@ -802,17 +797,13 @@ private:
   void putOpInFragWaitQue(Signal* signal);
   void putOverflowRecInFrag(Signal* signal);
   void putRecInFreeOverdir(Signal* signal);
-  void releaseDirectory(Signal* signal);
-  void releaseDirrange(Signal* signal);
   void releaseFsConnRec(Signal* signal);
   void releaseFsOpRec(Signal* signal);
   void releaseOpRec(Signal* signal);
   void releaseOverflowRec(Signal* signal);
   void releaseOverpage(Signal* signal);
   void releasePage(Signal* signal);
-  void releaseLogicalPage(Fragmentrec * fragP, Uint32 logicalPageId);
   void seizeDirectory(Signal* signal);
-  void seizeDirrange(Signal* signal);
   void seizeFragrec(Signal* signal);
   void seizeFsConnectRec(Signal* signal);
   void seizeFsOpRec(Signal* signal);
@@ -846,8 +837,6 @@ private:
 
   void zpagesize_error(const char* where);
 
-  void reportMemoryUsage(Signal* signal, int gth,
-                         BlockReference ref = CMVMI_REF);
   void reenable_expand_after_redo_log_exection_complete(Signal*);
 
   // charsets
@@ -865,27 +854,9 @@ private:
 
   // Variables
 /* --------------------------------------------------------------------------------- */
-/* DIRECTORY RANGE                                                                   */
+/* DIRECTORY                                                                         */
 /* --------------------------------------------------------------------------------- */
-  DirRange *dirRange;
-  DirRangePtr expDirRangePtr;
-  DirRangePtr gnsDirRangePtr;
-  DirRangePtr newDirRangePtr;
-  DirRangePtr rdDirRangePtr;
-  DirRangePtr nciOverflowrangeptr;
-  Uint32 cdirrangesize;
-  Uint32 cfirstfreeDirrange;
-/* --------------------------------------------------------------------------------- */
-/* DIRECTORYARRAY                                                                    */
-/* --------------------------------------------------------------------------------- */
-  Directoryarray *directoryarray;
-  DirectoryarrayPtr expDirptr;
-  DirectoryarrayPtr rdDirptr;
-  DirectoryarrayPtr sdDirptr;
-  DirectoryarrayPtr nciOverflowDirptr;
-  Uint32 cdirarraysize;
-  Uint32 cdirmemory;
-  Uint32 cfirstfreedir;
+  DynArr256Pool   directoryPool;
 /* --------------------------------------------------------------------------------- */
 /* FRAGMENTREC. ALL INFORMATION ABOUT FRAMENT AND HASH TABLE IS SAVED IN FRAGMENT    */
 /*         REC  A POINTER TO FRAGMENT RECORD IS SAVED IN ROOTFRAGMENTREC FRAGMENT    */
@@ -965,8 +936,13 @@ private:
   Page8Ptr spPageptr;
   Uint32 cfirstfreepage;
   Uint32 cpagesize;
-  Uint32 cnoOfAllocatedPages;
   Uint32 cpageCount;
+  Uint32 cnoOfAllocatedPages;
+  Uint32 cnoOfAllocatedPagesMax;
+  Uint32 m_maxAllocPages; // == cpagesize * (100 - m_free_pct) / 100
+  Uint32 m_free_pct;
+  bool m_oom; // if cnoOfAllocatedPages > m_maxAllocPages
+
 /* --------------------------------------------------------------------------------- */
 /* ROOTFRAGMENTREC                                                                   */
 /*          DURING EXPAND FRAGMENT PROCESS, EACH FRAGMEND WILL BE EXPAND INTO TWO    */
