@@ -1,5 +1,5 @@
 /* 
-   Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include <xfs/xfs.h>
 #endif
 
+#include "Ndbfs.hpp"
 #include "AsyncFile.hpp"
 #include "PosixAsyncFile.hpp"
 
@@ -36,17 +37,8 @@
 
 #include <NdbTick.h>
 
-// use this to test broken pread code
-//#define HAVE_BROKEN_PREAD
-
-#ifdef HAVE_BROKEN_PREAD
-#undef HAVE_PWRITE
-#undef HAVE_PREAD
-#endif
-
 // For readv and writev
 #include <sys/uio.h>
-
 #include <dirent.h>
 
 PosixAsyncFile::PosixAsyncFile(SimulatedBlock& fs) :
@@ -54,43 +46,38 @@ PosixAsyncFile::PosixAsyncFile(SimulatedBlock& fs) :
   theFd(-1),
   use_gz(0)
 {
-  memset(&azf,0,sizeof(azf));
+  memset(&nzf,0,sizeof(nzf));
+  init_mutex();
 }
 
 int PosixAsyncFile::init()
 {
-  // Create write buffer for bigger writes
-  theWriteBufferSize = WRITEBUFFERSIZE;
-  theWriteBufferUnaligned = (char *) ndbd_malloc(theWriteBufferSize +
-                                                 NDB_O_DIRECT_WRITE_ALIGNMENT-1);
-  theWriteBuffer = (char *)
-    (((UintPtr)theWriteBufferUnaligned + NDB_O_DIRECT_WRITE_ALIGNMENT - 1) &
-     ~(UintPtr)(NDB_O_DIRECT_WRITE_ALIGNMENT - 1));
+  /*
+    Preallocate read and write buffers for ndbzio to workaround
+    default behaviour of alloc/free at open/close
+  */
+  const size_t read_size = ndbz_bufsize_read();
+  const size_t write_size = ndbz_bufsize_write();
 
-  azfBufferUnaligned= (Byte*)ndbd_malloc((AZ_BUFSIZE_READ+AZ_BUFSIZE_WRITE)
-                                         +NDB_O_DIRECT_WRITE_ALIGNMENT-1);
-
-  azf.inbuf= (Byte*)(((UintPtr)azfBufferUnaligned
+  nzfBufferUnaligned= ndbd_malloc(read_size + write_size +
+                                  NDB_O_DIRECT_WRITE_ALIGNMENT-1);
+  nzf.inbuf= (Byte*)(((UintPtr)nzfBufferUnaligned
                       + NDB_O_DIRECT_WRITE_ALIGNMENT - 1) &
                      ~(UintPtr)(NDB_O_DIRECT_WRITE_ALIGNMENT - 1));
+  nzf.outbuf= nzf.inbuf + read_size;
 
-  azf.outbuf= azf.inbuf + AZ_BUFSIZE_READ;
-
-  az_mempool.size = az_mempool.mfree = az_inflate_mem_size()+az_deflate_mem_size();
+  /* Preallocate inflate/deflate buffers for ndbzio */
+  nz_mempool.size = nz_mempool.mfree =
+    ndbz_inflate_mem_size() + ndbz_deflate_mem_size();
 
   ndbout_c("NDBFS/AsyncFile: Allocating %u for In/Deflate buffer",
-           (unsigned int)az_mempool.size);
-  az_mempool.mem = (char*) ndbd_malloc(az_mempool.size);
+           (unsigned int)nz_mempool.size);
+  nz_mempool.mem = (char*) ndbd_malloc(nz_mempool.size);
 
-  azf.stream.opaque= &az_mempool;
-
-  if (!theWriteBuffer) {
-    DEBUG(ndbout_c("AsyncFile::writeReq, Failed allocating write buffer"));
-    return -1;
-  }//if
+  nzf.stream.opaque= &nz_mempool;
 
   return 0;
-}//AsyncFile::init()
+}
 
 #ifdef O_DIRECT
 static char g_odirect_readbuf[2*GLOBAL_PAGE_SIZE -1];
@@ -103,7 +90,7 @@ int PosixAsyncFile::check_odirect_write(Uint32 flags, int& new_flags, int mode)
   int ret;
   char * bufptr = (char*)((UintPtr(g_odirect_readbuf)+(GLOBAL_PAGE_SIZE - 1)) & ~(GLOBAL_PAGE_SIZE - 1));
   while (((ret = ::write(theFd, bufptr, GLOBAL_PAGE_SIZE)) == -1) &&
-         (errno == EINTR));
+         (errno == EINTR)) {};
   if (ret == -1)
   {
     new_flags &= ~O_DIRECT;
@@ -129,7 +116,7 @@ int PosixAsyncFile::check_odirect_read(Uint32 flags, int &new_flags, int mode)
   int ret;
   char * bufptr = (char*)((UintPtr(g_odirect_readbuf)+(GLOBAL_PAGE_SIZE - 1)) & ~(GLOBAL_PAGE_SIZE - 1));
   while (((ret = ::read(theFd, bufptr, GLOBAL_PAGE_SIZE)) == -1) &&
-         (errno == EINTR));
+         (errno == EINTR)) {};
   if (ret == -1)
   {
     ndbout_c("%s Failed to read using O_DIRECT, disabling",
@@ -231,8 +218,14 @@ void PosixAsyncFile::openReq(Request *request)
     break;
     return;
   }
-  if(flags & FsOpenReq::OM_GZ)
-    use_gz= 1;
+  if (flags & FsOpenReq::OM_GZ)
+  {
+    use_gz = 1;
+  }
+  else
+  {
+    use_gz = 0;
+  }
 
   // allow for user to choose any permissionsa with umask
   const int mode = S_IRUSR | S_IWUSR |
@@ -316,7 +309,8 @@ no_odirect:
     FsReadWriteReq* req = (FsReadWriteReq*)signal->getDataPtrSend();
 
     Uint32 index = 0;
-    Uint32 block = refToBlock(request->theUserReference);
+    Uint32 block = refToMain(request->theUserReference);
+    Uint32 instance = refToInstance(request->theUserReference);
 
 #ifdef HAVE_XFS_XFS_H
     if(platform_test_xfs_fd(theFd))
@@ -355,7 +349,8 @@ no_odirect:
         req->data.pageData[0] = m_page_ptr.i + cnt;
 
         m_fs.EXECUTE_DIRECT(block, GSN_FSWRITEREQ, signal,
-                            FsReadWriteReq::FixedLength + 1);
+                            FsReadWriteReq::FixedLength + 1,
+                            instance);
         
         cnt++;
         size += request->par.open.page_size;
@@ -363,13 +358,14 @@ no_odirect:
   retry:
       off_t save_size = size;
       char* buf = (char*)m_page_ptr.p;
-      while(size > 0){
+      while(size > 0)
+      {
 #ifdef TRACE_INIT
         write_cnt++;
 #endif
         int n;
 	if(use_gz)
-          n= azwrite(&azf,buf,size);
+          n= ndbzwrite(&nzf, buf, size);
         else
           n= write(theFd, buf, size);
 	if(n == -1 && errno == EINTR)
@@ -378,7 +374,7 @@ no_odirect:
 	}
 	if(n == -1 || n == 0)
 	{
-          ndbout_c("azwrite|write returned %d: errno: %d my_errno: %d",n,errno,my_errno);
+          ndbout_c("ndbzwrite|write returned %d: errno: %d my_errno: %d",n,errno,my_errno);
 	  break;
 	}
 	size -= n;
@@ -490,7 +486,7 @@ no_odirect:
   if(use_gz)
   {
     int err;
-    if((err= azdopen(&azf, theFd, new_flags)) < 1)
+    if((err= ndbzdopen(&nzf, theFd, new_flags)) < 1)
     {
       ndbout_c("Stewart's brain broke: %d %d %s",
                err, my_errno, theFileName.c_str());
@@ -506,10 +502,11 @@ int PosixAsyncFile::readBuffer(Request *req, char *buf,
   req->par.readWrite.pages[0].size = 0;
   off_t seek_val;
 #if ! defined(HAVE_PREAD)
+  FileGuard guard(this);
   if(!use_gz)
   {
     while((seek_val= lseek(theFd, offset, SEEK_SET)) == (off_t)-1
-          && errno == EINTR);
+          && errno == EINTR) {};
     if(seek_val == (off_t)-1)
     {
       return errno;
@@ -518,8 +515,8 @@ int PosixAsyncFile::readBuffer(Request *req, char *buf,
 #endif
   if(use_gz)
   {
-    while((seek_val= azseek(&azf, offset, SEEK_SET)) == (off_t)-1
-          && errno == EINTR);
+    while((seek_val= ndbzseek(&nzf, offset, SEEK_SET)) == (off_t)-1
+          && errno == EINTR) {};
     if(seek_val == (off_t)-1)
     {
       return errno;
@@ -533,14 +530,14 @@ int PosixAsyncFile::readBuffer(Request *req, char *buf,
 
 #if  ! defined(HAVE_PREAD)
     if(use_gz)
-      return_value = azread(&azf, buf, size, &error);
+      return_value = ndbzread(&nzf, buf, size, &error);
     else
       return_value = ::read(theFd, buf, size);
 #else // UNIX
     if(!use_gz)
       return_value = ::pread(theFd, buf, size, offset);
     else
-      return_value = azread(&azf, buf, size, &error);
+      return_value = ndbzread(&nzf, buf, size, &error);
 #endif
     if (return_value == -1 && errno == EINTR) {
       DEBUG(ndbout_c("EINTR in read"));
@@ -549,13 +546,13 @@ int PosixAsyncFile::readBuffer(Request *req, char *buf,
       if (return_value == -1)
         return errno;
     }
-    else if (return_value < 1 && azf.z_eof!=1)
+    else if (return_value < 1 && nzf.z_eof!=1)
     {
-      if(my_errno==0 && errno==0 && error==0 && azf.z_err==Z_STREAM_END)
+      if(my_errno==0 && errno==0 && error==0 && nzf.z_err==Z_STREAM_END)
         break;
       DEBUG(ndbout_c("ERROR DURING %sRead: %d off: %d from %s",(use_gz)?"gz":"",size,offset,theFileName.c_str()));
       ndbout_c("ERROR IN PosixAsyncFile::readBuffer %d %d %d %d",
-               my_errno, errno, azf.z_err, error);
+               my_errno, errno, nzf.z_err, error);
       if(use_gz)
         return my_errno;
       return errno;
@@ -610,18 +607,19 @@ void PosixAsyncFile::readvReq(Request *request)
 #endif
 }
 
-int PosixAsyncFile::writeBuffer(const char *buf, size_t size, off_t offset,
-                                size_t chunk_size)
+int PosixAsyncFile::writeBuffer(const char *buf, size_t size, off_t offset)
 {
+  size_t chunk_size = 256*1024;
   size_t bytes_to_write = chunk_size;
   int return_value;
 
   m_write_wo_sync += size;
 
 #if ! defined(HAVE_PWRITE)
+  FileGuard guard(this);
   off_t seek_val;
   while((seek_val= lseek(theFd, offset, SEEK_SET)) == (off_t)-1
-	&& errno == EINTR);
+	&& errno == EINTR) {};
   if(seek_val == (off_t)-1)
   {
     return errno;
@@ -637,12 +635,12 @@ int PosixAsyncFile::writeBuffer(const char *buf, size_t size, off_t offset,
 
 #if ! defined(HAVE_PWRITE)
     if(use_gz)
-      return_value= azwrite(&azf, buf, bytes_to_write);
+      return_value= ndbzwrite(&nzf, buf, bytes_to_write);
     else
       return_value = ::write(theFd, buf, bytes_to_write);
 #else // UNIX
     if(use_gz)
-      return_value= azwrite(&azf, buf, bytes_to_write);
+      return_value= ndbzwrite(&nzf, buf, bytes_to_write);
     else
       return_value = ::pwrite(theFd, buf, bytes_to_write, offset);
 #endif
@@ -651,7 +649,7 @@ int PosixAsyncFile::writeBuffer(const char *buf, size_t size, off_t offset,
       DEBUG(ndbout_c("EINTR in write"));
     } else if (return_value == -1 || return_value < 1){
       ndbout_c("ERROR IN PosixAsyncFile::writeBuffer %d %d %d",
-               my_errno, errno, azf.z_err);
+               my_errno, errno, nzf.z_err);
       if(use_gz)
         return my_errno;
       return errno;
@@ -686,17 +684,17 @@ void PosixAsyncFile::closeReq(Request *request)
   }
   int r;
   if(use_gz)
-    r= azclose(&azf);
+    r= ndbzclose(&nzf);
   else
     r= ::close(theFd);
   use_gz= 0;
   Byte *a,*b;
-  a= azf.inbuf;
-  b= azf.outbuf;
-  memset(&azf,0,sizeof(azf));
-  azf.inbuf= a;
-  azf.outbuf= b;
-  azf.stream.opaque = (void*)&az_mempool;
+  a= nzf.inbuf;
+  b= nzf.outbuf;
+  memset(&nzf,0,sizeof(nzf));
+  nzf.inbuf= a;
+  nzf.outbuf= b;
+  nzf.stream.opaque = (void*)&nz_mempool;
 
   if (-1 == r) {
 #ifndef DBUG_OFF
@@ -737,7 +735,7 @@ void PosixAsyncFile::appendReq(Request *request)
   while(size > 0){
     int n;
     if(use_gz)
-      n= azwrite(&azf,buf,size);
+      n= ndbzwrite(&nzf,buf,size);
     else
       n= write(theFd, buf, size);
     if(n == -1 && errno == EINTR){
@@ -771,69 +769,82 @@ void PosixAsyncFile::removeReq(Request *request)
   }
 }
 
-void PosixAsyncFile::rmrfReq(Request *request, char *path, bool removePath)
+void
+PosixAsyncFile::rmrfReq(Request *request, const char * src, bool removePath)
 {
-  Uint32 path_len = strlen(path);
-  Uint32 path_max_copy = PATH_MAX - path_len;
-  char* path_add = &path[path_len];
-
-  if(!request->par.rmrf.directory){
+  if(!request->par.rmrf.directory)
+  {
     // Remove file
-    if(unlink((const char *)path) != 0 && errno != ENOENT)
+    if(unlink(src) != 0 && errno != ENOENT)
       request->error = errno;
     return;
   }
-  // Remove directory
-  DIR* dirp = opendir((const char *)path);
-  if(dirp == 0){
+
+  char path[PATH_MAX];
+  strcpy(path, src);
+  strcat(path, "/");
+
+  DIR* dirp;
+  struct dirent * dp;
+loop:
+  dirp = opendir(path);
+  if(dirp == 0)
+  {
     if(errno != ENOENT)
       request->error = errno;
     return;
   }
-  struct dirent * dp;
-  while ((dp = readdir(dirp)) != NULL){
-    if ((strcmp(".", dp->d_name) != 0) && (strcmp("..", dp->d_name) != 0)) {
-      BaseString::snprintf(path_add, (size_t)path_max_copy, "%s%s",
-	       DIR_SEPARATOR, dp->d_name);
-      if(remove((const char*)path) == 0){
-        path[path_len] = 0;
-	continue;
-      }
 
-      rmrfReq(request, path, true);
-      path[path_len] = 0;
-      if(request->error != 0){
-	closedir(dirp);
-	return;
+  while ((dp = readdir(dirp)) != NULL)
+  {
+    if ((strcmp(".", dp->d_name) != 0) && (strcmp("..", dp->d_name) != 0)) 
+    {
+      int len = strlen(path);
+      strcat(path, dp->d_name);
+      if (remove(path) == 0)
+      {
+        path[len] = 0;
+        continue;
       }
+      
+      closedir(dirp);
+      strcat(path, "/");
+      goto loop;
     }
   }
   closedir(dirp);
-  if(removePath && rmdir((const char *)path) != 0){
+  path[strlen(path)-1] = 0; // remove /
+  if (strcmp(src, path) != 0)
+  {
+    char * t = strrchr(path, '/');
+    t[1] = 0;
+    goto loop;
+  }
+
+  if(removePath && rmdir(src) != 0)
+  {
     request->error = errno;
   }
-  return;
 }
 
-void PosixAsyncFile::endReq()
+PosixAsyncFile::~PosixAsyncFile()
 {
-  // Thread is ended with return
-  if (theWriteBufferUnaligned)
-    ndbd_free(theWriteBufferUnaligned, theWriteBufferSize);
+  /* Free the read and write buffer memory used by ndbzio */
+  if (nzfBufferUnaligned)
+    ndbd_free(nzfBufferUnaligned,
+              ndbz_bufsize_read() +
+              ndbz_bufsize_write() +
+              NDB_O_DIRECT_WRITE_ALIGNMENT-1);
+  nzfBufferUnaligned = NULL;
 
-  if (azfBufferUnaligned)
-    ndbd_free(azfBufferUnaligned, (AZ_BUFSIZE_READ*AZ_BUFSIZE_WRITE)
-              +NDB_O_DIRECT_WRITE_ALIGNMENT-1);
+  /* Free the inflate/deflate buffers for ndbzio */
+  if(nz_mempool.mem)
+    ndbd_free(nz_mempool.mem, nz_mempool.size);
+  nz_mempool.mem = NULL;
 
-  if(az_mempool.mem)
-    ndbd_free(az_mempool.mem,az_mempool.size);
-
-  az_mempool.mem = NULL;
-  theWriteBufferUnaligned = NULL;
-  azfBufferUnaligned = NULL;
+  destroy_mutex();
 }
 
-
 void PosixAsyncFile::createDirectories()
 {
   char* tmp;
