@@ -1,4 +1,5 @@
-/* Copyright (C) 2003 MySQL AB
+/*
+   Copyright (c) 2005, 2010, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,16 +12,17 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #include <ndb_global.h>
-#include <my_sys.h>
 #include <mgmapi.h>
+#include <mgmapi_internal.h>
 
 #include <NdbOut.hpp>
 #include <Properties.hpp>
-#include <socket_io.h>
 #include <InputStream.hpp>
+#include <NdbTick.h>
 
 #include <debugger/EventLogger.hpp>
 #include <kernel/NodeBitmask.hpp>
@@ -28,7 +30,8 @@
 #include "ndb_logevent.hpp"
 
 extern
-int ndb_mgm_listen_event_internal(NdbMgmHandle, const int filter[], int);
+int ndb_mgm_listen_event_internal(NdbMgmHandle, const int filter[],
+                                  int, NDB_SOCKET_TYPE*);
 
 struct ndb_logevent_error_msg {
   enum ndb_logevent_handle_error code;
@@ -49,30 +52,60 @@ struct ndb_logevent_handle {
   enum ndb_logevent_handle_error m_error;
 };
 
-extern "C"
+/*
+  Create a new NdbLogEventHandle for reading events from
+  the same socket as the NdbMgmHandle
+*/
+
 NdbLogEventHandle
-ndb_mgm_create_logevent_handle(NdbMgmHandle mh,
-			       const int filter[])
+ndb_mgm_create_logevent_handle_same_socket(NdbMgmHandle mh)
 {
-  int fd= ndb_mgm_listen_event_internal(mh, filter, 1);
-
-  if (fd == -1)
-    return 0;
-
   NdbLogEventHandle h=
-    (NdbLogEventHandle)my_malloc(sizeof(ndb_logevent_handle),MYF(MY_WME));
+    (NdbLogEventHandle)malloc(sizeof(ndb_logevent_handle));
+  if (!h)
+    return NULL;
 
-  h->socket= fd;
+  h->socket= _ndb_mgm_get_socket(mh);
 
   return h;
 }
 
 extern "C"
+NdbLogEventHandle
+ndb_mgm_create_logevent_handle(NdbMgmHandle mh,
+			       const int filter[])
+{
+  NdbLogEventHandle h=
+    (NdbLogEventHandle)malloc(sizeof(ndb_logevent_handle));
+  if (!h)
+    return NULL;
+
+  NDB_SOCKET_TYPE sock;
+  if(ndb_mgm_listen_event_internal(mh, filter, 1, &sock) < 0)
+  {
+    free(h);
+    return 0;
+  }
+
+  h->socket= sock;
+
+  return h;
+}
+
+extern "C"
+#ifdef NDB_WIN
+SOCKET
+ndb_logevent_get_fd(const NdbLogEventHandle h)
+{
+  return h->socket.s;
+}
+#else
 int
 ndb_logevent_get_fd(const NdbLogEventHandle h)
 {
-  return h->socket;
+  return h->socket.fd;
 }
+#endif
 
 extern "C"
 void ndb_mgm_destroy_logevent_handle(NdbLogEventHandle * h)
@@ -81,9 +114,9 @@ void ndb_mgm_destroy_logevent_handle(NdbLogEventHandle * h)
     return;
 
   if ( *h )
-    close((*h)->socket);
+    my_socket_close((*h)->socket);
 
-  my_free((char*)* h,MYF(MY_ALLOW_ZERO_PTR));
+  free(*h);
   * h = 0;
 }
 
@@ -262,6 +295,12 @@ struct Ndb_logevent_body_row ndb_logevent_body[]= {
   ROW( MemoryUsage, "pages_total",  4, pages_total),
   ROW( MemoryUsage, "block",        5, block),
 
+  ROW( MTSignalStatistics, "mt_deliver_thread", 1, thr_no),
+  ROW( MTSignalStatistics, "mt_prioa_count", 2, prioa_count),
+  ROW( MTSignalStatistics, "mt_prioa_size", 3, prioa_size),
+  ROW( MTSignalStatistics, "mt_priob_count", 4, priob_count),
+  ROW( MTSignalStatistics, "mt_priob_size", 5, priob_size),
+
       /* ERROR */
   ROW( TransporterError, "to_node", 1, to_node),
   ROW( TransporterError, "code",    2, code),
@@ -358,6 +397,26 @@ struct Ndb_logevent_body_row ndb_logevent_body[]= {
 
   ROW( SingleUser,          "type",	     1, type),
   ROW( SingleUser,          "node_id",	     2, node_id),
+
+  ROW( LogFileInitStatus,   "node_id",       1, node_id ),
+  ROW( LogFileInitStatus,   "total_files",   2, total_files),
+  ROW( LogFileInitStatus,   "file_done",     3, file_done),
+  ROW( LogFileInitStatus,   "total_mbytes",  4, total_mbytes),
+  ROW( LogFileInitStatus,   "mbytes_done",   5, mbytes_done),
+
+ 
+  ROW( RedoStatus,          "log_part",      1, log_part), 
+  ROW( RedoStatus,          "head_file_no",  2, head_file_no), 
+  ROW( RedoStatus,          "head_mbyte",    3, head_mbyte), 
+  ROW( RedoStatus,          "tail_file_no",  4, tail_file_no), 
+  ROW( RedoStatus,          "tail_mbyte",    5, tail_mbyte), 
+  ROW( RedoStatus,          "total_hi",      6, total_hi), 
+  ROW( RedoStatus,          "total_lo",      7, total_lo), 
+  ROW( RedoStatus,          "free_hi",       8, free_hi), 
+  ROW( RedoStatus,          "free_lo",       9, free_lo), 
+  ROW( RedoStatus,          "no_logfiles",  10, no_logfiles),
+  ROW( RedoStatus,          "logfilesize",  11, logfilesize),
+ 
   { NDB_LE_ILLEGAL_TYPE, 0, 0, 0, 0, 0}
 };
 
@@ -442,11 +501,15 @@ int ndb_logevent_get_next(const NdbLogEventHandle h,
 
   SocketInputStream in(h->socket, timeout_in_milliseconds);
 
-  Properties p;
-  char buf[256];
-
-  /* header */
-  while (1) {
+  /*
+    Read log event header until header received
+    or timeout expired. The MGM server will continusly
+    send <PING>'s that should be ignored.
+  */
+  char buf[1024];
+  NDB_TICKS start = NdbTick_CurrentMillisecond();
+  while(1)
+  {
     if (in.gets(buf,sizeof(buf)) == 0)
     {
       h->m_error= NDB_LEH_READ_ERROR;
@@ -466,9 +529,14 @@ int ndb_logevent_get_next(const NdbLogEventHandle h,
 
     if(in.timedout())
         return 0;
-  }
 
-  /* read name-value pairs into properties object */
+    if ((NdbTick_CurrentMillisecond() - start) > timeout_in_milliseconds)
+      return 0;
+
+  };
+
+  /* Read name-value pairs until empty new line */
+  Properties p;
   while (1)
   {
     if (in.gets(buf,sizeof(buf)) == 0)
@@ -530,18 +598,40 @@ int ndb_logevent_get_next(const NdbLogEventHandle h,
   /* fill in header info from p */
   for (i= 0; ndb_logevent_body[i].token; i++)
   {
-    if ( ndb_logevent_body[i].type != dst->type )
-      continue;
-    if ( p.get(ndb_logevent_body[i].token, &val) == 0 )
+    if ( ndb_logevent_body[i].type == dst->type )
+      break;
+  }
+
+  if (ndb_logevent_body[i].token)
+  {
+    do {
+
+      if ( p.get(ndb_logevent_body[i].token, &val) == 0 )
+      {
+        h->m_error= NDB_LEH_UNKNOWN_EVENT_VARIABLE;
+        return -1;
+      }
+      if ( memcpy_atoi((char *)dst+ndb_logevent_body[i].offset, val,
+                       ndb_logevent_body[i].size) )
+      {
+        h->m_error= NDB_LEH_INTERNAL_ERROR;
+        return -1;
+      }
+    } while (ndb_logevent_body[++i].type == dst->type);
+  }
+  else
+  {
+    if (!p.get("data", &val))
     {
       h->m_error= NDB_LEH_UNKNOWN_EVENT_VARIABLE;
       return -1;
     }
-    if ( memcpy_atoi((char *)dst+ndb_logevent_body[i].offset, val,
-		     ndb_logevent_body[i].size) )
+    BaseString tmp(val);
+    Vector<BaseString> list;
+    tmp.split(list);
+    for (unsigned j = 0; j<list.size(); j++)
     {
-      h->m_error= NDB_LEH_INTERNAL_ERROR;
-      return -1;
+      dst->Data[j] = atoi(list[j].c_str());
     }
   }
   return 1;
