@@ -26,29 +26,12 @@
 static const char* _dbname = "TEST_DB";
 static int g_loops = 7;
 
-
-NDB_STD_OPTS_VARS;
-
-static struct my_option my_long_options[] =
+struct my_option my_long_options[] =
 {
   NDB_STD_OPTS("ndb_desc"),
   { 0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}
 };
 
-static void usage()
-{
-  ndb_std_print_version();
-}
-#if 0
-static my_bool
-get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
-	       const char *argument)
-{
-  return ndb_std_get_one_option(optid, opt, argument ? argument :
-				"d:t:O,/tmp/testBitfield.trace");
-}
-#endif
-
 static const NdbDictionary::Table* create_random_table(Ndb*);
 static int transactions(Ndb*, const NdbDictionary::Table* tab);
 static int unique_indexes(Ndb*, const NdbDictionary::Table* tab);
@@ -61,7 +44,7 @@ int
 main(int argc, char** argv){
   NDB_INIT(argv[0]);
   const char *load_default_groups[]= { "mysql_cluster",0 };
-  load_defaults("my",load_default_groups,&argc,&argv);
+  ndb_load_defaults(NULL, load_default_groups,&argc,&argv);
   int ho_error;
 
   if ((ho_error=handle_options(&argc, &argv, my_long_options,
@@ -77,7 +60,7 @@ main(int argc, char** argv){
       return NDBT_ProgramExit(res);
   }
   
-  Ndb_cluster_connection con(opt_connect_str);
+  Ndb_cluster_connection con(opt_ndb_connectstring, opt_ndb_nodeid);
   if(con.connect(12, 5, 1))
   {
     return NDBT_ProgramExit(NDBT_FAILED);
@@ -87,7 +70,7 @@ main(int argc, char** argv){
   Ndb* pNdb;
   pNdb = new Ndb(&con, _dbname);  
   pNdb->init();
-  while (pNdb->waitUntilReady() != 0);
+  while (pNdb->waitUntilReady() != 0) {};
 
   NdbDictionary::Dictionary * dict = pNdb->getDictionary();
 
@@ -148,7 +131,9 @@ create_random_table(Ndb* pNdb)
   do {
     NdbDictionary::Table tab;
     Uint32 cols = 1 + (rand() % (NDB_MAX_ATTRIBUTES_IN_TABLE - 1));
-    Uint32 length = 4090;
+    const Uint32 maxLength = 4090;
+    Uint32 length = maxLength;
+    Uint8  defbuf[(maxLength + 7)/8];
     
     BaseString name; 
     name.assfmt("TAB_%d", rand() & 65535);
@@ -171,6 +156,10 @@ create_random_table(Ndb* pNdb)
       col.setType(NdbDictionary::Column::Bit);
       
       Uint32 len = 1 + (rand() % (length - 1));
+      memset(defbuf, 0, (length + 7)/8);
+      for (Uint32 j = 0; j < len/8; j++)
+        defbuf[j] = 0x63;
+      col.setDefaultValue(defbuf, (len + 7)/8);
       col.setLength(len); length -= len;
       int nullable = (rand() >> 16) & 1;
       col.setNullable(nullable); length -= nullable;
@@ -692,4 +681,3 @@ testBitmask()
 }
 
 template class Vector<Alloc>;
-template class Vector<Uint32>;
